#!/usr/bin/env python
"""Run nosetests with build/lib.* in sys.path"""

# Fix up sys.path so as to include the correct build/lib.*/ directory.
import sys
from distutils.dist import Distribution
from distutils.command.build import build

build_cmd = build(Distribution({"ext_modules": True}))
build_cmd.finalize_options()
lib_dirn = build_cmd.build_lib
sys.path.insert(0, lib_dirn)

# Dump info plugin stuff
import nose
import logging
from nose.plugins import Plugin

logger = logging.getLogger("nose.plugins.pylibmc")

def dump_infos():
    logger.info("injected path: %s", lib_dirn)
    import pylibmc, _pylibmc
    if hasattr(_pylibmc, "__file__"):
        logger.info("loaded _pylibmc from %s", _pylibmc.__file__)
        if not _pylibmc.__file__.startswith(lib_dirn):
            logger.warn("double-check the source path")
    else:
        logger.warn("static _pylibmc: %s", _pylibmc)
    logger.info("libmemcached version: %s", _pylibmc.libmemcached_version)
    logger.info("pylibmc version: %s", _pylibmc.__version__)
    logger.info("support compression: %s", _pylibmc.support_compression)
    logger.info("support sasl auth: %s", _pylibmc.support_sasl)

class PylibmcVersionDumper(Plugin):
    name = "info"
    enableOpt = "info"

    def __init__(self):
        Plugin.__init__(self)

    def begin(self):
        dump_infos()

if __name__ == "__main__":
    nose.main(addplugins=[PylibmcVersionDumper()])



"""Tests. They want YOU!!"""

import unittest
import pylibmc
from pylibmc.test import make_test_client

class PylibmcTestCase(unittest.TestCase):
    memcached_client = pylibmc.Client
    memcached_server_type = "tcp"
    memcached_host = "127.0.0.1"
    memcached_port = "11211"

    def setUp(self):
        self.mc = make_test_client(cls=self.memcached_client,
                                   server_type=self.memcached_server_type,
                                   host=self.memcached_host,
                                   port=self.memcached_port)

    def tearDown(self):
        self.mc.disconnect_all()
        del self.mc

def dump_infos():
    if hasattr(_pylibmc, "__file__"):
        print "Starting tests with _pylibmc at", _pylibmc.__file__
    else:
        print "Starting tests with static _pylibmc:", _pylibmc
    print "Reported libmemcached version:", _pylibmc.libmemcached_version
    print "Reported pylibmc version:", _pylibmc.__version__
    print "Support compression:", _pylibmc.support_compression



from nose.tools import eq_
from pylibmc.client import translate_server_spec
import _pylibmc

def test_translate():
    eq_(translate_server_spec("111.122.133.144"),
        (_pylibmc.server_type_tcp, "111.122.133.144", 11211))

def test_translate():
    eq_(translate_server_spec("111.122.133.144:5555"),
        (_pylibmc.server_type_tcp, "111.122.133.144", 5555))

def test_udp_translate():
    eq_(translate_server_spec("udp:199.299.399.499:5555"),
        (_pylibmc.server_type_udp, "199.299.399.499", 5555))

def test_udp_translate_ipv6():
    eq_(translate_server_spec("udp:[abcd:abcd::1]:5555"),
        (_pylibmc.server_type_udp, "abcd:abcd::1", 5555))



from __future__ import with_statement

import Queue
import pylibmc
from nose.tools import eq_, ok_
from tests import PylibmcTestCase

class PoolTestCase(PylibmcTestCase):
    pass

class ClientPoolTests(PoolTestCase):
    def test_simple(self):
        p = pylibmc.ClientPool(self.mc, 2)
        with p.reserve() as smc:
            ok_(smc)
            ok_(smc.set("a", 1))
            eq_(smc["a"], 1)

    def test_exhaust(self):
        p = pylibmc.ClientPool(self.mc, 2)
        with p.reserve() as smc1:
            with p.reserve() as smc2:
                self.assertRaises(Queue.Empty, p.reserve().__enter__)

# TODO Thread-mapped pool tests



"""Ported cmemcached tests"""

import pylibmc
from nose.tools import eq_
from tests import PylibmcTestCase

class TestCmemcached(PylibmcTestCase):
    def testSetAndGet(self):
        self.mc.set("num12345", 12345)
        eq_(self.mc.get("num12345"), 12345)
        self.mc.set("str12345", "12345")
        eq_(self.mc.get("str12345"), "12345")

    def testDelete(self):
        self.mc.set("str12345", "12345")
        #delete return True on success, otherwise False
        assert self.mc.delete("str12345")
        assert self.mc.get("str12345") is None

        # This test only works with old memcacheds. This has become a "client
        # error" in memcached.
        try:
            assert not self.mc.delete("hello world")
        except pylibmc.ClientError:
            pass

    def testGetMulti(self):
        self.mc.set("a", "valueA")
        self.mc.set("b", "valueB")
        self.mc.set("c", "valueC")
        result = self.mc.get_multi(["a", "b", "c", "", "hello world"])
        eq_(result, {'a':'valueA', 'b':'valueB', 'c':'valueC'})

    def testBigGetMulti(self):
        count = 10 ** 4
        keys = ['key%d' % i for i in xrange(count)]
        pairs = zip(keys, ['value%d' % i for i in xrange(count)])
        for key, value in pairs:
            self.mc.set(key, value)
        result = self.mc.get_multi(keys)
        eq_(result, dict(pairs))

    def testFunnyDelete(self):
        assert not self.mc.delete("")

    def testAppend(self):
        self.mc.delete("a")
        self.mc.set("a", "I ")
        assert self.mc.append("a", "Do")
        eq_(self.mc.get("a"), "I Do")

    def testPrepend(self):
        self.mc.delete("a")
        self.mc.set("a", "Do")
        assert self.mc.prepend("a", "I ")
        eq_(self.mc.get("a"), "I Do")



"""Snappy libmemcached wrapper

pylibmc is a Python wrapper around TangentOrg's libmemcached library.

The interface is intentionally made as close to python-memcached as possible,
so that applications can drop-in replace it.

Example usage
=============

Create a connection and configure it::

    >>> import pylibmc
    >>> m = pylibmc.Client(["10.0.0.1"], binary=True)
    >>> m.behaviors = {"tcp_nodelay": True, "ketama": True}

Nevermind this doctest shim::

    >>> from pylibmc.test import make_test_client
    >>> mc = make_test_client(behaviors=m.behaviors)

Basic operation::

    >>> mc.set("some_key", "Some value")
    True
    >>> value = mc.get("some_key")
    >>> value
    'Some value'
    >>> mc.set("another_key", 3)
    True
    >>> mc.delete("another_key")
    True
    >>> mc.set("key", "1")  # str or int is fine
    True

Atomic increments and decrements::

    >>> mc.incr("key")
    2L
    >>> mc.decr("key")
    1L

Batch operation::

    >>> mc.get_multi(["key", "another_key"])
    {'key': '1'}
    >>> mc.set_multi({"cats": ["on acid", "furry"], "dogs": True})
    []
    >>> mc.get_multi(["cats", "dogs"])
    {'cats': ['on acid', 'furry'], 'dogs': True}
    >>> mc.delete_multi(["cats", "dogs", "nonextant"])
    False
    >>> mc.add_multi({"cats": ["on acid", "furry"], "dogs": True})
    []
    >>> mc.get_multi(["cats", "dogs"])
    {'cats': ['on acid', 'furry'], 'dogs': True}
    >>> mc.add_multi({"cats": "not set", "dogs": "definitely not set", "bacon": "yummy"})
    ['cats', 'dogs']
    >>> mc.get_multi(["cats", "dogs", "bacon"])
    {'cats': ['on acid', 'furry'], 'bacon': 'yummy', 'dogs': True}
    >>> mc.delete_multi(["cats", "dogs", "bacon"])
    True

Further Reading
===============

See http://sendapatch.se/projects/pylibmc/
"""

import _pylibmc
from .consts import hashers, distributions
from .client import Client
from .pools import ClientPool, ThreadMappedPool

support_compression = _pylibmc.support_compression
support_sasl = _pylibmc.support_sasl

__version__ = _pylibmc.__version__
__all__ = ["hashers", "distributions", "Client",
           "ClientPool", "ThreadMappedPool"]



import pylibmc
from pylibmc.test import make_test_client
from tests import PylibmcTestCase
from nose.tools import eq_

class ClientTests(PylibmcTestCase):
    def test_zerokey(self):
        bc = make_test_client(binary=True)
        k = "\x00\x01"
        assert bc.set(k, "test")
        rk = bc.get_multi([k]).keys()[0]
        eq_(k, rk)
